@@ -5,9 +5,11 @@ from signalsdk.iot_mqtt_connection import IotMqttConnection
 from signalsdk.local_mqtt import OnEventReceived, LocalMqtt
 from signalsdk.api import get_local_config
 from signalsdk.application_config import get_config_for_app
+from signalsdk.config import LOCALMQTT_SDK_TOPIC_PREFIX
 from signalsdk.signal_exception import SignalAppCallBackError, SignalAppConfigEnvError, \
-    SignalAppInitError, SignalDeviceDomainNotFoundError, \
+    SignalAppInitError, \
     SignalDeploymentNotFoundError
+from .validator import throw_if_parameter_not_found_in
 
 class SignalApp:
     def __init__(self):
@@ -32,10 +34,8 @@ class SignalApp:
                      f"{str(delta)}")
 
         self.deviceApiDomainName = delta.get("deviceApiDomainName") or self.deviceApiDomainName
-        logging.info(f"signalsdk::deviceApiDomainName: {self.deviceApiDomainName}")
 
-        if not self.deviceApiDomainName:
-            raise SignalDeviceDomainNotFoundError
+        throw_if_parameter_not_found_in(self.deviceApiDomainName,'device api','device shadow')
 
         self.deviceDeploymentVersion = delta.get("deploymentVersion")
         logging.info(f"signalsdk::device deploymentVersion: {self.deviceDeploymentVersion}")
@@ -65,12 +65,14 @@ class SignalApp:
                                               self.account_id,
                                               self.app_id,
                                               self.deviceApiDomainName)
-
+        current_subtopic = self.localMqtt.get_subscribed_topic()
         if "sdkSubTopic" in self.appSettings["settingsForSDK"] and \
-            self.appSettings["settingsForSDK"]["sdkSubTopic"] != {}:
-            desired_subtopic = self.appSettings["settingsForSDK"]["sdkSubTopic"]
-            current_subtopic = self.localMqtt.get_subscribed_topic()
-            self.renew_topic_subscription(current_subtopic, desired_subtopic)
+            self.appSettings["settingsForSDK"]["sdkSubTopic"]:
+            desired_subtopic = LOCALMQTT_SDK_TOPIC_PREFIX \
+                               + self.appSettings["settingsForSDK"]["sdkSubTopic"]
+        else:
+            desired_subtopic = LOCALMQTT_SDK_TOPIC_PREFIX + self.app_id
+        self.renew_topic_subscription(current_subtopic, desired_subtopic)
 
         if self.appSettings["settingsForApp"] != {}:
             try:
@@ -106,6 +108,7 @@ class SignalApp:
         self.thing_name = self.appConfig['thingName']
         self.account_id = self.appConfig['accountId']
         self.app_id = os.getenv('APPLICATION_ID')
+        throw_if_parameter_not_found_in(self.app_id, 'application id', 'environment variables')
         self.shadow_client = None
         if not self.app_id:
             raise SignalAppConfigEnvError
@@ -129,29 +132,32 @@ class SignalApp:
         self.shadow_client.get_application_config()
         self.shadow_client.start_listening_device_shadow()
 
-    def next(self, event):
+    def next(self, event, next_app_id=''):
         """Publish the event
         :param event:
         :return:
         """
-        if 'settingsForSDK' not in self.appSettings:
-            logging.info(f"signalsdk: settingForSDK not exists: {self.appSettings}"
+        if 'settingsForSDK' not in self.appSettings and not next_app_id:
+            logging.info(f"signalsdk:next called without topic to publish to: {self.appSettings}"
                          f"event: {event} ")
             return
         logging.info(f"signalsdk: forwarding event to topic: "
-                     f"{self.appSettings['settingsForSDK']} "
                      f"event: {event}")
 
         if not self.shadow_client:
             logging.error("signalsdk: shadow client is not created")
             raise SignalAppInitError
+
         if 'sdkPubTopic' in self.appSettings['settingsForSDK'] and \
-                self.appSettings['settingsForSDK'] != {}:
-            logging.info(f"signalsdk next() publishing sdk config: "
-                         f"{self.appSettings['settingsForSDK']}")
-            self.localMqtt.publish(self.appSettings['settingsForSDK']['sdkPubTopic'], event)
-        else:
-            logging.info("signalsdk next() called but no publish topic found in sdk config")
+                self.appSettings['settingsForSDK']['sdkPubTopic']:
+            topic = LOCALMQTT_SDK_TOPIC_PREFIX + self.appSettings['settingsForSDK']['sdkPubTopic']
+            logging.info(f"signalsdk next() publishing to sdk topic: "
+                         f"{topic}")
+            self.localMqtt.publish(topic, event)
+        elif next_app_id:
+            topic = LOCALMQTT_SDK_TOPIC_PREFIX + next_app_id
+            logging.info(f"signalsdk next() publishing to applicationId topic: {topic}")
+            self.localMqtt.publish(topic, event)
 
     def report_configuration_change(self):
         logging.info("signalsdk: report_configuration_change")
@@ -172,6 +178,8 @@ class SignalApp:
         self.shadow_client.update_application_config(to_report)
 
     def renew_topic_subscription(self, current_topic, desired_topic):
+        logging.info(f"signalsdk:renew_topic_subscription current_topic: {current_topic} "
+                     f"desired_topic: {desired_topic}")
         if current_topic and current_topic != desired_topic:
             self.localMqtt.unsubscribe()
         if desired_topic and current_topic != desired_topic:
